/*
 Copyright (c) 2012-2015, Pierre-Olivier Latour
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 * The name of Pierre-Olivier Latour may not be used to endorse
 or promote products derived from this software without specific
 prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL PIERRE-OLIVIER LATOUR BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

var ENTER_KEYCODE = 13;

var _path = null;
var _pendingReloads = [];
var _reloadingDisabled = 0;
var _items = {};
var _selectedItems = {};
var _downloadItems = [];
var _deleteItems = [];
var _isDownloading = false;

function PVItem(path, size, name) {
  this.path = path;
  this.size = size;
  this.name = name;
}

function numAttrs(obj) {
  var count = 0;
  for(var key in obj) {
    if (obj.hasOwnProperty(key)) {
      ++count;
    }
  }
  return count;
}

function areAllItemsSelected() {
  var totalItemsCount = numAttrs(_items);
  var selectedItemsCount = numAttrs(_selectedItems);
  console.log("areAllItemsSelected: totalItemsCount:" + totalItemsCount + ", selectedItemsCount:" + selectedItemsCount);
  if (totalItemsCount == selectedItemsCount) {
    return true;
  } else {
    return false;
  }
}

function formatFileSize(bytes) {
  if (bytes >= 1000000000) {
    return (bytes / 1000000000).toFixed(2) + ' GB';
  }
  if (bytes >= 1000000) {
    return (bytes / 1000000).toFixed(2) + ' MB';
  }
  return (bytes / 1000).toFixed(2) + ' KB';
}

function _showError(message, textStatus, errorThrown) {
  if (message != null && message != "") {
		$("#alerts").prepend(tmpl("template-alert", {
			level: "danger",
			title: (errorThrown != "" ? errorThrown : textStatus) + ": ",
			description: message
		}));
  } else {
		$("#alerts").prepend(tmpl("template-alert", {
			level: "danger",
			title: (errorThrown != "" ? errorThrown : textStatus),
			description: message
		}));
	}
}

function showTransferError() {
    var alerts = $("#alerts div");
    var alert_count = alerts.length;
    if (alert_count == 0) {
        var transfer_error_text = $("#transfer_error").text();
        _showError("", transfer_error_text, "");
    }
}

function _disableReloads() {
  _reloadingDisabled += 1;
}

function _enableReloads() {
  _reloadingDisabled -= 1;
  
  if (_pendingReloads.length > 0) {
    _reload(_pendingReloads.shift());
  }
}

var lazy;
function createLazy() {
  $(function () {
    lazy = $('.lazy').lazy({
      visibleOnly: true,
      defaultImage: null
    });
  });
}

function updateBatchButtons() {
  var selectedItemsCount = numAttrs(_selectedItems);
  if (selectedItemsCount > 0) {
    $("#button-download-all").prop("disabled", false);
    $("#button-delete-all").prop("disabled", false);
  } else {
    $("#button-download-all").prop("disabled", true);
    $("#button-delete-all").prop("disabled", true);
  }
}

var zipManager = (function () {
  var zipFileEntry = null;
  var zipWriter, writer;
  var zipPVItems;
  var itemReader;
  var RequestFileSystem = window.webkitRequestFileSystem || window.mozRequestFileSystem || window.requestFileSystem;
  var cancelled = false;

  function createTempFile(callback) {
    var tmpFilename = "tmp.zip";
    RequestFileSystem(window.TEMPORARY, 4 * 1024 * 1024 * 1024, function(filesystem) {
      function create() {
        filesystem.root.getFile(tmpFilename, {
          create : true
        }, function(zipFile) {
          callback(zipFile);
        });
      }

      filesystem.root.getFile(tmpFilename, null, function(entry) {
        entry.remove(create, create);
      }, create);
    });
  }

  return {
    addFiles: function addFiles(items, oninit, onadd, onprogress, onend, onerror) {
      cancelled = false;
      var addIndex = 0;
      zipPVItems = items;

      function nextFile() {
        if (cancelled) {
          onend(cancelled);
          return;
        }
        var item = zipPVItems[addIndex];
        onadd(item);
        itemReader = new zip.PVItemReader(item);
        zipWriter.add(item.name,
          itemReader,
          function () { /* onend */
            addIndex++;
            if (addIndex < zipPVItems.length)
              nextFile();
            else
              onend(cancelled);
          },
          onprogress);
      }

      function createZipWriter() {
        var dontDeflate = true;
        zip.useWebWorkers = false;
        zip.createWriter(writer,
          function (writer) { /* callback */
            zipWriter = writer;
            oninit();
            nextFile();
          },
          function () { /* onerror */
            console.log("zip writer error");
            onerror("");
            onend(true);
          },
          dontDeflate);
      }

      if (zipWriter)
        nextFile();
      else {
        if (RequestFileSystem != undefined) {
          createTempFile(function(fileEntry) {
            zipFileEntry = fileEntry;
            writer = new zip.FileWriter(zipFileEntry);
            console.log("create FileWriter");
            createZipWriter();
          });
        } else {
          writer = new zip.BlobWriter();
          console.log("create BlobWriter");
          createZipWriter();
        }
      }
    },
    getBlobURL: function (callback) {
      zipWriter.close(function (blob) {
        var blobURL;
        if (zipFileEntry == null) {
          blobURL = URL.createObjectURL(blob);
        } else {
          blobURL = zipFileEntry.toURL();
        }
        callback(blobURL);
        zipWriter = null;
      });
    },
    getBlob: function (callback) {
      zipWriter.close(callback);
    },
    cancelDownload: function () {
      cancelled = true;
      if (itemReader) {
        itemReader.close();
        itemReader = null;
      }
    },
    closeWriter: function () {
      zipWriter.close(function (blob) {
        zipWriter = null;
      });
    }
  };
})();

function downloadAll() {
  $("#download-all-modal").modal("show");

  var i = -1;
  _downloadItems = [];
  for (var item in _selectedItems) {
    if (_selectedItems.hasOwnProperty(item)) {
      // do stuff
      _downloadItems.push(_selectedItems[item]);
    }
  }

  zipManager.addFiles(_downloadItems,
    function () {
      /* oninit */
    }, function (item) {
      console.log("zip: add " + item.name);
      /* onadd */
      $("#download-all-filename").text(item.name);
      i += 1;
      var total_progress = parseInt(i * 100 / _downloadItems.length);
      $("#download-all-progress").css('width', total_progress+'%').attr('aria-valuenow', total_progress);
    }, function (index, size) {
      /* onprogress */
      console.log("zip: progress " + index + "/" + size);
      if (index == 0) {
        $("#download-all-file-progress").text("");
      } else {
        var file_progress = parseInt(index * 100 / size);
        $("#download-all-file-progress").text(file_progress + "%");
      }
    }, function (cancelled) {
      /* onend */
      _isDownloading = false;
      if (cancelled) {
        $("#download-all-modal").modal("hide");
        zipManager.closeWriter();
        return;
      }
      var total_progress = 100;
      $("#download-all-progress").css('width', total_progress+'%').attr('aria-valuenow', total_progress);
      console.log("zip: end");
      zipManager.getBlobURL(function(blobURL) {
        var a = document.createElement("a");
        document.body.appendChild(a);
        a.style = "display: none";
        a.href = blobURL;
        var app_name = $("#app_name").text();
        console.log("app name = " + app_name);
        var zip_name = app_name + ".zip";
        a.download = zip_name;
        a.click();
        setTimeout(function(){
          $("#download-all-modal").modal("hide");
          document.body.removeChild(a);
          URL.revokeObjectURL(blobURL);
        }, 100);
      });
    }, function (message){
      // onerror
      showTransferError();
    });
}

function cancelDownloadAll() {
  zipManager.cancelDownload();
  _isDownloading = false;
}

var deletaAllCancelled = false;
var deleteIndex = 0;
var deleteAllOnProgress;
var deleteAllOnEnd;

function deletePVItem(item, callback) {
  console.log("deletePVItem: " + item.path);
  $.ajax({
    url: 'delete',
    type: 'POST',
    data: {path: item.path},
    dataType: 'json'
  }).done(function() {
    console.log( "success" );
  }).fail(function() {
    console.log( "error" );
  }).always(function() {
    console.log( "complete" );
    callback();
  });
}

function deleteNextFile() {
  console.log("deleteItem nextFile deleteIndex = " + deleteIndex);
  if (deletaAllCancelled) {
    deleteAllOnEnd();
    return;
  }
  var item = _deleteItems[deleteIndex];
  deletePVItem(item, function () {
    console.log("deleteItem callback deleteIndex = " + deleteIndex);
    deleteIndex++;
    deleteAllOnProgress(deleteIndex, _deleteItems.length);
    if (deleteIndex < _deleteItems.length)
      deleteNextFile();
    else
      deleteAllOnEnd();
  });

}

function deleteAll(onprogress, onend) {
  deletaAllCancelled = false;
  deleteIndex = 0;
  _deleteItems = [];
  for (var item in _selectedItems) {
    if (_selectedItems.hasOwnProperty(item)) {
      // do stuff
      _deleteItems.push(_selectedItems[item]);
    }
  }

  deleteAllOnProgress = onprogress;
  deleteAllOnEnd = onend;

  deleteNextFile();
}

function cancelDeleteAll() {
  deletaAllCancelled = true;
}

function _reload(path) {
  if (_reloadingDisabled) {
    if ($.inArray(path, _pendingReloads) < 0) {
      _pendingReloads.push(path);
    }
    return;
  }
  
  _disableReloads();
  $.ajax({
    url: 'list',
    type: 'GET',
    data: {path: path},
    dataType: 'json'
  }).fail(function(jqXHR, textStatus, errorThrown) {
    showTransferError();
  }).done(function(data, textStatus, jqXHR) {
    var scrollPosition = $(document).scrollTop();
    
    if (path != _path) {
      $("#path").empty();
      if (path == "/") {
        $("#path").append('<li class="active">' + _device + '</li>');
      } else {
        $("#path").append('<li data-path="/"><a>' + _device + '</a></li>');
        var components = path.split("/").slice(1, -1);
        for (var i = 0; i < components.length - 1; ++i) {
          var subpath = "/" + components.slice(0, i + 1).join("/") + "/";
          $("#path").append('<li data-path="' + subpath + '"><a>' + components[i] + '</a></li>');
        }
        $("#path > li").click(function(event) {
          _reload($(this).data("path"));
          event.preventDefault();
        });
        $("#path").append('<li class="active">' + components[components.length - 1] + '</li>');
      }
      _path = path;
    }
    
    $("#listing").empty();
    _items = {};
    _selectedItems = {};
    $("#checkbox-all").prop('checked', false);
    console.log("list " + data.length + " items");
    for (var i = 0, file; file = data[i]; ++i) {
      var item = new PVItem(file.path, file.size, file.name);
      _items[file.path] = item;
      $(tmpl("template-listing", file)).data(file).appendTo("#listing");
    }
    
    $(".edit").editable(function(value, settings) { 
      var name = $(this).parent().parent().data("name");
      if (value != name) {
        var path = $(this).parent().parent().data("path");
        $.ajax({
          url: 'move',
          type: 'POST',
          data: {oldPath: path, newPath: _path + value},
          dataType: 'json'
        }).fail(function(jqXHR, textStatus, errorThrown) {
          _showError("Failed moving \"" + path + "\" to \"" + _path + value + "\"", textStatus, errorThrown);
        }).always(function() {
          _reload(_path);
        });
      }
      return value;
    }, {
      onedit: function(settings, original) {
        _disableReloads();
      },
      onsubmit: function(settings, original) {
        _enableReloads();
      },
      onreset: function(settings, original) {
        _enableReloads();
      },
      tooltip: 'Click to rename...'
    });
    
    $(".button-download").click(function(event) {
      var path = $(this).parent().parent().data("path");
      setTimeout(function() {
        window.location = "download?path=" + encodeURIComponent(path);
      }, 0);
    });
    
    $("#button-download-all").click(function(event) {
      event.stopPropagation();
      if (_isDownloading == true) {
        return;
      } else {
        _isDownloading = true;
      }
      var RequestFileSystem = window.webkitRequestFileSystem || window.mozRequestFileSystem || window.requestFileSystem;
      if (RequestFileSystem == undefined) {
        $("#download-all-not-support").modal("show");
        return;
      }

      downloadAll();
    });

    $("#button-download-all-cancel").click(function(event) {
      cancelDownloadAll();
    });

    $("#button-download-all-modal").click(function (event) {
      $("#download-all-not-support").modal("hide");
      downloadAll();
    });

    $("#button-delete-all").click(function(event) {
      event.stopImmediatePropagation();
      console.log("button-delete-all");
      $("#delete-all-modal").modal("show");
      deleteAll(function (index, total) {
        //onprogress
        var total_progress = parseInt(index * 100 / total);
        $("#delete-all-progress").css('width', total_progress+'%').attr('aria-valuenow', total_progress);
      }, function () {
        //onend
        _reload("/");
        $("#delete-all-modal").modal("hide");
      });
    });

    $("#button-delete-all-cancel").click(function(event) {
      cancelDeleteAll();
    });

    $(".button-open").click(function(event) {
      var path = $(this).parent().parent().data("path");
      _reload(path);
    });
    
    $(".button-move").click(function(event) {
      var path = $(this).parent().parent().data("path");
      if (path[path.length - 1] == "/") {
        path = path.slice(0, path.length - 1);
      }
      $("#move-input").data("path", path);
      $("#move-input").val(path);
      $("#move-modal").modal("show");
    });
    
    $(".button-delete").click(function(event) {
      var path = $(this).parent().parent().data("path");
      $.ajax({
        url: 'delete',
        type: 'POST',
        data: {path: path},
        dataType: 'json'
      }).fail(function(jqXHR, textStatus, errorThrown) {
        _showError("Failed deleting \"" + path + "\"", textStatus, errorThrown);
      }).always(function() {
        _reload(_path);
      });
    });

    // check box of a single item
    $(".checkbox").change(function() {
      if (this.id != "checkbox-all") {
        var path = $(this).parent().parent().data("path");
        if ($(this).is(":checked")) {
          var item = _items[path];
          _selectedItems[path] = item;
        } else {
          delete _selectedItems[path];
        }

        // update select all checked state
        if (areAllItemsSelected()) {
          $("#checkbox-all").prop('checked', true);
        } else {
          $("#checkbox-all").prop('checked', false);
        }

        updateBatchButtons();
      }
    });

    // select all check box
    $("#checkbox-all").change(function(event) {
      event.stopImmediatePropagation();
      var newCheckValue;
      if ($(this).is(":checked")) {
        newCheckValue = true;
      } else {
        newCheckValue = false;
      }

      var rows = $("tr.row-file");
      //console.log("rows count = " + rows.length);

      rows.each(function(i, tr) {
        var checkbox = $(".checkbox", tr);
        var path = tr.id;
        if (newCheckValue == true) {
          var item = _items[path];
          _selectedItems[path] = item;
        } else {
          delete _selectedItems[path];
        }
        //console.log("checkbox = " + newCheckValue);
        $(".checkbox", tr).prop("checked", newCheckValue);
      });

      updateBatchButtons();
    });

    $(document).scrollTop(scrollPosition);

    //lazy load
    createLazy();
    /*$(function() {
      $('.lazy').lazy({
        visibleOnly: true,
        defaultImage:null
      });
    });*/

    updateBatchButtons();

  }).always(function() {
    _enableReloads();
  });
}



$(document).ready(function() {
  $("#button-download-all").prop("disabled", true);
  $("#button-delete-all").prop("disabled", true);
  
  // Workaround Firefox and IE not showing file selection dialog when clicking on "upload-file" <button>
  // Making it a <div> instead also works but then it the button doesn't work anymore with tab selection or accessibility
  $("#upload-file").click(function(event) {
    $("#fileupload").click();
  });
  
  // Prevent event bubbling when using workaround above
  $("#fileupload").click(function(event) {
    event.stopPropagation();
  });
  
  $("#fileupload").fileupload({
    dropZone: $(document),
    pasteZone: null,
    autoUpload: true,
    sequentialUploads: true,
    limitConcurrentUploads: 1,
    // forceIframeTransport: true,
    
    url: 'upload',
    type: 'POST',
    dataType: 'json',
    
    start: function(e) {
      console.log("upload start");
      $(".uploading").show();
    },
    
    stop: function(e) {
      console.log("upload stop");
      $(".uploading").hide();
      setTimeout(function () {
        _reload(_path);
      }, 300);
    },
    
    add: function(e, data) {
      var file = data.files[0];
      var url = "can_upload?size=" + file.size + "&name=" + encodeURIComponent(file.name);
      $.getJSON(url, function (result) {
				var resultString = result.result_string;
				if (resultString != "true") {
					_showError(file.name, resultString, "");
				} else {
					data.formData = {
						path: _path
					};
					data.context = $(tmpl("template-uploads", {
						path: _path + file.name
					})).appendTo("#uploads");
					var jqXHR = data.submit();
					data.context.find("button").click(function (event) {
						jqXHR.abort();
					});
				}
			}).done(function () {

			}).fail(function () {
              showTransferError();
            }).always(function () {

            });

    },
    
    progress: function(e, data) {
      var progress = parseInt(data.loaded / data.total * 100, 10);
      data.context.find(".progress-bar").css("width", progress + "%");
    },
    
    done: function(e, data) {
      //_reload(_path);
    },
    
    fail: function(e, data) {
      var file = data.files[0];
      if (data.errorThrown != "abort") {
        _showError("Failed uploading \"" + file.name + "\" to \"" + _path + "\"", data.textStatus, data.errorThrown);
      }
    },
    
    always: function(e, data) {
      data.context.remove();
    },
    
  });
  
  $("#create-input").keypress(function(event) {
    if (event.keyCode == ENTER_KEYCODE) {
      $("#create-confirm").click();
    };
  });
  
  $("#create-modal").on("shown.bs.modal", function(event) {
    $("#create-input").focus();
    $("#create-input").select();
  });
  
  $("#create-folder").click(function(event) {
    $("#create-input").val("Untitled folder");
    $("#create-modal").modal("show");
  });
  
  $("#create-confirm").click(function(event) {
    $("#create-modal").modal("hide");
    var name = $("#create-input").val();
    if (name != "") {
      $.ajax({
        url: 'create',
        type: 'POST',
        data: {path: _path + name},
        dataType: 'json'
      }).fail(function(jqXHR, textStatus, errorThrown) {
        _showError("Failed creating folder \"" + name + "\" in \"" + _path + "\"", textStatus, errorThrown);
      }).always(function() {
        _reload(_path);
      });
    }
  });
  
  $("#move-input").keypress(function(event) {
    if (event.keyCode == ENTER_KEYCODE) {
      $("#move-confirm").click();
    };
  });
  
  $("#move-modal").on("shown.bs.modal", function(event) {
    $("#move-input").focus();
    $("#move-input").select();
  })
  
  $("#move-confirm").click(function(event) {
    $("#move-modal").modal("hide");
    var oldPath = $("#move-input").data("path");
    var newPath = $("#move-input").val();
    if ((newPath != "") && (newPath[0] == "/") && (newPath != oldPath)) {
      $.ajax({
        url: 'move',
        type: 'POST',
        data: {oldPath: oldPath, newPath: newPath},
        dataType: 'json'
      }).fail(function(jqXHR, textStatus, errorThrown) {
        _showError("Failed moving \"" + oldPath + "\" to \"" + newPath + "\"", textStatus, errorThrown);
      }).always(function() {
        _reload(_path);
      });
    }
  });
  
  $("#reload").click(function(event) {
    $("#alerts").empty();
    _reload(_path);
  });
  
  _reload("/");
  
});
